// ============================================================================
// Project:	Deductive Database
// Filename:	atom.h
// Purpose:	Atoms (Symbolic Constants, Enumeration Type Constants)
// Last Change:	07.09.2015
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "str.h"
#include "check.h"

#include "atom.h"


//=============================================================================
// Constructor and Destructor:
//=============================================================================


//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

atom_c::atom_c()
{
	// Constructor should never be called (use init instead, see atomtab_c)
	CHECK_IMPOSSIBLE("atom_c::constructor: SHould not be called");

	// Initialize attributes:
	Name[0] = '\0';
	Id = -1;
	HashNext = ATOM_NULL;

	// Set Magic number:
	//CHECK_CODE(Magic = ATOM_MAGIC);
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

// atom_c::~atom_c()
// {
//	// Check this object:
//	CHECK_VALID("atom_c::destructor");
//
//	// Make this object invalid:
//	CHECK_CODE(Magic = 0);
// }


//=============================================================================
// Object Methods:
//=============================================================================


//-----------------------------------------------------------------------------
// init: Initialize object (used in place of the constructor).
//-----------------------------------------------------------------------------

void atom_c::init(str_t name_start, str_t name_end, int atom_id, atom_t next)
{
	// Check parameters:
	CHECK(name_start != STR_NULL, "atom_c::init: name_start is NULL");
	CHECK(name_end != STR_NULL, "atom_c::init: name_end is NULL");
	CHECK(name_start < name_end,
			"atom_c::init: name_start not before name_end");
	CHECK(atom_id >= 0, "atom_c::init: atom_id is negative");
	CHECK_PAR_NULLOK(next, "atom_c::init");

	// Copy atom name (maximally VER_MAX_ATOMSIZE characters):
	str_t from = name_start;
	char *to = Name;
	int n = VER_MAX_ATOMSIZE;
	while(from != name_end && n-- > 0)
		*to++ = *from++;
	*to++ = '\0';

	// Set other attributes:
	Id = atom_id;
	HashNext = next;

	// Set Magic number:
	CHECK_CODE(Magic = ATOM_MAGIC);
}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
str_t	atom_c::check() const
{
	// Check magic number:
	if(Magic != ATOM_MAGIC)
		return "wrong magic number";

	// Id must be not negative:
	if(Id < 0)
		return "Atom ID is negative";

	// Name must be non-empty:
	if(Name[0] == '\0')
		return "Atom name is empty";

	// Ok:
	return 0;
}

#endif

